// CS370 - Fall 2016
// Lab09 - TimeCube

#ifdef OSX
	#include <GLUT/glut.h>
#else
	#include <GL/glew.h>
	#include <GL/glut.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Shader file utility functions
#include "shaderutils.h"

// Shader files
GLchar* vertexFile = "basicvert.vs";
GLchar* fragmentFile = "basicfrag.fs";

// Shader objects
GLuint shaderProg;

#define X 0
#define Y 1
#define Z 2
#define CUBE 1
#define DEG_PER_SEC (360.0f/60.0f)
#define FPS_INTERVAL 1000

// Cube vertices
GLfloat cube[][3] = {{-0.5f,-0.5f,-0.5f},{0.5f,-0.5f,-0.5f},{0.5f,-0.5f,0.5f},
                     {-0.5f,-0.5f,0.5f},{-0.5f,0.5f,-0.5f},{0.5f,0.5f,-0.5f},
                     {0.5f,0.5f,0.5f},{-0.5f,0.5f,0.5f}};

// Global camera vectors
GLfloat eye[3] = { 0.0f,0.707f,0.707f };
GLfloat at[3] = { 0.0f,0.0f,0.0f };
GLfloat up[3] = { 0.0f,1.0f,0.0f };

// Global screen extents
GLint ww;
GLint hh;

// Animation variables
GLfloat theta = 0.0f;
GLint fps = 25;
GLint frame = 0;
GLint time = 0;
GLint lasttime = 0;
GLint fpstime = 0;
GLfloat rpm = 10.0f;
GLint revs = 0;

// Text strings
char fpsStr[32];
char des_fpsStr[32];
char revStr[32];

void display();
void render_Scene();
void keyfunc(unsigned char key, int x, int y);
void idlefunc();
void reshape(int w, int h);
void create_lists();
static void mytimer(int v);
void colorcube();
void quad(GLfloat v1[], GLfloat v2[], GLfloat v3[], GLfloat v4[]);
void draw_text();

int main(int argc, char *argv[])
{
	// Initialize GLUT
	glutInit(&argc,argv);

	// Initialize the window with double buffering and RGB colors
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	// Set the window size to image size
	glutInitWindowSize(500,500);

	// Create window
	glutCreateWindow("Time Cube");

#ifndef OSX
	// Initialize GLEW - MUST BE DONE AFTER CREATING GLUT WINDOW
	glewInit();
#endif
	
	// Define callbacks
	glutDisplayFunc(display);
	glutKeyboardFunc(keyfunc);
	glutReshapeFunc(reshape);

	// TODO: Timer/Idle callbacks

	// Create display lists
	create_lists();

	// Set background color to white
	glClearColor(1.0f,1.0f,1.0f,1.0f);

	// Enable depth test
	glEnable(GL_DEPTH_TEST);


	// Load shader programs
	shaderProg = load_shaders(vertexFile,fragmentFile);
	glUseProgram(shaderProg);

	// Begin event loop
	glutMainLoop();

	return 0;
}

// Display callback
void display()
{
	GLfloat ratio;

	// Reset background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set 3D projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	GLfloat xratio = 1.0f;
	GLfloat yratio = 1.0f;
	// Taller than wide so scale height
	if (ww <= hh)
	{
		yratio = (GLfloat)ww / (GLfloat)hh;
	}
	// Wider than tall so scale width
	else
	{
		xratio = (GLfloat)hh / (GLfloat)ww;
	}
	glOrtho(-2.0f*xratio, 2.0*xratio, -2.0f*yratio, 2.0f*yratio, -2.0f, 2.0f);

	// Set modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(eye[X], eye[Y], eye[Z], at[X], at[Y], at[Z], up[X], up[Y], up[Z]);


	// Render scene
	render_Scene();

	// Draw text
	draw_text();
	
	// Flush buffer
	glFlush();

	// Swap buffers
	glutSwapBuffers();
}

// Scene render function
void render_Scene()
{
	// Draw color cube (via display list)
	glPushMatrix();
		glTranslatef(0.0f, 0.0f, 0.0f);
		glRotatef(theta, 0.0f, 1.0f, 0.0f);
		glScalef(1.0f, 1.0f, 1.0f);
		glCallList(CUBE);
	glPopMatrix();
}

// Routine to draw text on screen
void draw_text()
{
	// Increment frame counter
	frame++;

	// Get elapsed time
	time = glutGet(GLUT_ELAPSED_TIME);

	// Update fps text every FPS_INTERVAL
	if (time - fpstime > FPS_INTERVAL)
	{
		// TODO: Create strings

		fpstime = time;
		frame = 0;
	}

	// Save previous matrices, switch to 2D, and turn off lighting for text
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(-1.0,1.0,-1.0,1.0);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	// Switch to default shader
	glUseProgram(0);
	
	// TODO: Set fps text position to top left and draw
	glColor3f(1.0f,1.0f,0.0f);

	// TODO: Set desired fps text position under fps and draw
	glColor3f(1.0f,1.0f,0.0f);

	// TODO: Set revs text position beside fps and draw
	glColor3f(0.0f, 0.0f, 1.0f);

	// Restore previous matricies and turn on lighting
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	
	// Switch to shader program
	glUseProgram(shaderProg);
}

// Keyboard callback
void keyfunc(unsigned char key, int x, int y)
{
	// + - change desired fps
	if (key == '+' || key == '=')
	{
		fps++;
	}
	else if (key == '-' || key == '_')
	{
		fps--;
		if (fps < 1)
		{
			fps = 1;
		}
	}

	// <esc> quits
	if (key == 27)
	{
		exit(0);
	}

	// NOTE: DO NOT REDRAW SCREEN HERE!
}

// Timer callback
static void mytimer(int v)
{
	// TODO: Compute angular change for desired rpm

	// Render scene
	glutPostRedisplay();

	// TODO: Start new timer

}

// Idle callback
void idlefunc()
{
	// TODO: Get total elapsed time

	// TODO: Update if past desired interval
		// Compute angular change for desired rpm
		
		// Completed full revolution

		// Update lasttime (reset timer)

		// Render scene

}

// Reshape callback
void reshape(int w, int h)
{
	// Set new screen extents
	glViewport(0,0,w,h);

	// Store new extents
	ww = w;
	hh = h;
}

void create_lists()
{
	glNewList(CUBE, GL_COMPILE);
		colorcube();
	glEndList();
}

// Routine to draw cube
void colorcube()
{
	// Top face
	glColor3f(1.0f,0.0f,0.0f);
	quad(cube[4],cube[7],cube[6],cube[5]);

	// Bottom face
	glColor3f(1.0f,0.0f,0.0f);
	quad(cube[0],cube[1],cube[2],cube[3]);

	// Left face
	glColor3f(0.0f,1.0f,0.0f);
	quad(cube[0],cube[3],cube[7],cube[4]);

	// Right face
	glColor3f(0.0f,1.0f,0.0f);
	quad(cube[1],cube[5],cube[6],cube[2]);

	// Front face
	glColor3f(0.0f,0.0f,1.0f);
	quad(cube[2],cube[6],cube[7],cube[3]);

	// Back face
	glColor3f(0.0f,0.0f,1.0f);
	quad(cube[0],cube[4],cube[5],cube[1]);
}

// Routine to draw (outlined) quadrilateral face
void quad(GLfloat v1[], GLfloat v2[], GLfloat v3[], GLfloat v4[])
{

	// Draw face with per vertex normals
	glBegin(GL_POLYGON);
		glVertex3fv(v1);
		glVertex3fv(v2);
		glVertex3fv(v3);
		glVertex3fv(v4);
	glEnd();

	// Draw outline
	glColor3f(0.0f,0.0f,0.0f);
	glBegin(GL_LINE_LOOP);
		glVertex3fv(v1);
		glVertex3fv(v2);
		glVertex3fv(v3);
		glVertex3fv(v4);
	glEnd();
}
